import React from 'react';
import ReactDOM from 'react-dom';
import { ApolloClient, createHttpLink, InMemoryCache, ApolloProvider } from '@apollo/client';

var REHYDRATION_STATE_DATA_KEY = "__APOLLO_STATE__";
var ROOT_CONTAINER_ID = "app-root";

/**
 * Render universal application main component with server side prefetched state in client side
 * @param {ReactElement} appElement: Application main React Element
 * @param {Object} inMemoryCacheConfig apollo client InMemoryCache constructor config object (available options: https://www.apollographql.com/docs/react/advanced/caching/#configuration) - optional
 */

function clientRender(appElement, inMemoryCacheConfig) {
  // create apollo client
  var client = new ApolloClient({
    link: createHttpLink({
      uri: "/graphql",
      credentials: "same-origin"
    }),
    cache: new InMemoryCache(inMemoryCacheConfig).restore(window[REHYDRATION_STATE_DATA_KEY]),
    defaultOptions: {
      watchQuery: {
        errorPolicy: "all"
      },
      query: {
        errorPolicy: "all"
      },
      mutate: {
        errorPolicy: "all"
      }
    }
  }); // wrapping main component with Apollo Provider

  var app = React.createElement(ApolloProvider, {
    client: client
  }, appElement); // render to page

  ReactDOM.hydrate(app, document.getElementById(ROOT_CONTAINER_ID));
}

export default clientRender;
//# sourceMappingURL=clientRender.js.map
